package ui.view.main;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ItemEvent;

import javax.swing.Box;
import javax.swing.GrayFilter;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;

import interfaces.GraphEditable;
import ui.controller.Control;
import utility.ImageImport;

public class Inspector extends JSplitPane {
	private final UnitGraph unitGraph;
	private final InspectorTable table;
	private final Control control;
	private final JScrollPane scrollGraph = new JScrollPane();
	private final JPanel graphLabel = new JPanel();
	private final JLabel maxGraph = new JLabel("100%");
	private final JLabel medGraph = new JLabel("50%");
	private final JLabel minGraph = new JLabel("0%");
	private final JToolBar toolBarGraph = new JToolBar();
	private String[] comboContext = { "", "5", "10", "20", "100", "1000" };
	private JComboBox<String> localPeriodInput = new JComboBox<String>(comboContext);
	private JButton resetButton = new JButton("", new ImageIcon(ImageImport.loadImage("/Images/resetIcon3.png")));
	private final ImageIcon localPeriodButtonImageEnabled = new ImageIcon(
			ImageImport.loadImage("/Images/Graph.png"));
	
	private final ImageIcon localPeriodButtonImageDisabled = new ImageIcon(
			GrayFilter.createDisabledImage(ImageImport.loadImage("/Images/Graph.png")));
	private JButton localPeriodButton = new JButton("", localPeriodButtonImageEnabled);
	
	public Inspector(Control control) {
		this.control = control;
		table = new InspectorTable(control);
		unitGraph = new UnitGraph(control);
		initUI();
		
	}
	private void initUI() {
		this.setOrientation(JSplitPane.VERTICAL_SPLIT);
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		this.setTopComponent(scrollPane);
		this.setBottomComponent(scrollGraph);
		this.setDividerLocation(1000);
		
		scrollGraph.setViewportView(unitGraph);
		// Set up of the Properties section


		graphLabel.setLayout(new BorderLayout(0, 10));
		graphLabel.add(maxGraph, BorderLayout.NORTH);
		graphLabel.add(medGraph, BorderLayout.CENTER);
		graphLabel.add(minGraph, BorderLayout.SOUTH);

		toolBarGraph.setFloatable(false);
		toolBarGraph.setAlignmentY(Component.RIGHT_ALIGNMENT);

		localPeriodButton.setToolTipText("Toggle Local/Global Mode");
		toolBarGraph.add(localPeriodButton);
		// ComboBox
		localPeriodInput.setEditable(true);
		localPeriodInput.setVisible(false);
		localPeriodInput.setMaximumSize(new Dimension(20, 23));
		localPeriodInput.addItemListener(aListener -> {
			if (aListener.getStateChange() == ItemEvent.DESELECTED) {
				validateInput(localPeriodInput.getEditor().getItem().toString(), true);
			}

		});

		toolBarGraph.add(localPeriodInput);

		// localPeriodButtonFunction
		localPeriodButton.addActionListener(actionEvent -> {
			boolean newState = !localPeriodInput.isVisible();
			changeLocalPeriodButtonAppeareance(newState);
			unitGraph.setUseLocalPeriod(newState);
		});

		toolBarGraph.add(Box.createHorizontalGlue());
		resetButton.setToolTipText("Reset");
		resetButton.addActionListener(actionEvent -> unitGraph.reset());
		toolBarGraph.add(resetButton);
		scrollGraph.setRowHeaderView(graphLabel);
		scrollGraph.setColumnHeaderView(toolBarGraph);
		
	}
	
	/**
	 * Validate the LocalMode Input and when its valid save on the Element.
	 * 
	 * @param text         the inputText to validate.
	 * @param bShowMessage when true, open a MessageDialog when text invalid.
	 */
	private void validateInput(String text, boolean bShowMessage) {
		int localPeriodInputValue;
		try {
			localPeriodInputValue = Integer.parseInt(text);
		} catch (NumberFormatException e) {
			if (bShowMessage)
				JOptionPane.showMessageDialog(null,
						'"' + text + '"' + " is not a valid Input. \n Use whole numbers.");
			return;
		}
		unitGraph.setLocalPeriod(localPeriodInputValue);
	}
	
	/**
	 * This Method updates the UnitGraph, saves the old LocalModeState and load the
	 * new LocalModeState.
	 * 
	 * @param element The new Element to load the UnitGraph
	 */
	private void updateUnitGraph(GraphEditable element) {
		// SaveOld LocalMode State.
		if (localPeriodInput.isVisible()) {
			// Save Old State
			validateInput(localPeriodInput.getEditor().getItem().toString(), false);
		}
		// Update UnitGraph
		unitGraph.initNewElement(element);
		// Load LocalMode State.
		changeLocalPeriodButtonAppeareance(unitGraph.isUsingLocalPeriod());
		localPeriodInput.getEditor().setItem(unitGraph.getLocalPeriod());
	}

	/**
	 * Displayed the actual LocalModeState.
	 * 
	 * @param enabled
	 */
	private void changeLocalPeriodButtonAppeareance(boolean enabled) {
		localPeriodInput.setVisible(enabled);
		if (enabled) {
			localPeriodButton.setIcon(localPeriodButtonImageEnabled);
		} else {
			localPeriodButton.setIcon(localPeriodButtonImageDisabled);
		}
	}

}
